@@ -12,6 +12,7 @@
 
 import logging
 import os
+from datetime import datetime, timezone
 from logging.config import fileConfig
 
 import argparse
@@ -19,7 +20,13 @@ from botocore.config import Config
 from configparser import ConfigParser
 
 from common.schedulers.slurm_commands import get_nodes_info, set_nodes_down
-from slurm_plugin.common import CONFIG_FILE_DIR, InstanceManager, print_with_count, retrieve_instance_type_mapping
+from slurm_plugin.common import (
+    CONFIG_FILE_DIR,
+    InstanceManager,
+    is_clustermgtd_heartbeat_valid,
+    print_with_count,
+    retrieve_instance_type_mapping,
+)
 
 log = logging.getLogger(__name__)
 
@@ -29,12 +36,14 @@ class SlurmResumeConfig:
         "max_retry": 1,
         "max_batch_size": 500,
         "update_node_address": True,
+        "clustermgtd_timeout": 300,
         "proxy": "NONE",
         "logging_config": os.path.join(os.path.dirname(__file__), "logging", "parallelcluster_resume_logging.conf"),
         "hosted_zone": None,
         "dns_domain": None,
         "use_private_hostname": False,
         "instance_type_mapping": "/opt/slurm/etc/pcluster/instance_name_type_mappings.json",
+        "all_or_nothing_batch": False,
     }
 
     def __init__(self, config_file_path):
@@ -63,18 +72,27 @@ class SlurmResumeConfig:
         self.use_private_hostname = config.getboolean(
             "slurm_resume", "use_private_hostname", fallback=self.DEFAULTS.get("use_private_hostname")
         )
-        self.master_private_ip = config.get("slurm_resume", "master_private_ip")
-        self.master_hostname = config.get("slurm_resume", "master_hostname")
+        self.head_node_private_ip = config.get("slurm_resume", "master_private_ip")
+        self.head_node_hostname = config.get("slurm_resume", "master_hostname")
         self.max_batch_size = config.getint(
             "slurm_resume", "max_batch_size", fallback=self.DEFAULTS.get("max_batch_size")
         )
         self.update_node_address = config.getboolean(
             "slurm_resume", "update_node_address", fallback=self.DEFAULTS.get("update_node_address")
         )
+        self.all_or_nothing_batch = config.getboolean(
+            "slurm_resume", "all_or_nothing_batch", fallback=self.DEFAULTS.get("all_or_nothing_batch")
+        )
         instance_name_type_mapping_file = config.get(
             "slurm_resume", "instance_type_mapping", fallback=self.DEFAULTS.get("instance_type_mapping")
         )
         self.instance_name_type_mapping = retrieve_instance_type_mapping(instance_name_type_mapping_file)
+        self.clustermgtd_timeout = config.getint(
+            "slurm_resume",
+            "clustermgtd_timeout",
+            fallback=self.DEFAULTS.get("clustermgtd_timeout"),
+        )
+        self.clustermgtd_heartbeat_file_path = config.get("slurm_resume", "clustermgtd_heartbeat_file_path")
 
         # Configure boto3 to retry 1 times by default
         self._boto3_retry = config.getint("slurm_resume", "boto3_retry", fallback=self.DEFAULTS.get("max_retry"))
@@ -111,6 +129,19 @@ def _handle_failed_nodes(node_list):
 
 def _resume(arg_nodes, resume_config):
     """Launch new EC2 nodes according to nodes requested by slurm."""
+    # Check heartbeat
+    current_time = datetime.now(tz=timezone.utc)
+    if not is_clustermgtd_heartbeat_valid(
+        current_time, resume_config.clustermgtd_timeout, resume_config.clustermgtd_heartbeat_file_path
+    ):
+        log.error(
+            "No valid clustermgtd heartbeat detected, clustermgtd is down!\n"
+            "Please check clustermgtd log for error.\n"
+            "Not launching nodes %s",
+            arg_nodes,
+        )
+        _handle_failed_nodes(arg_nodes)
+        return
     log.info("Launching EC2 instances for the following Slurm nodes: %s", arg_nodes)
     node_list = [node.name for node in get_nodes_info(arg_nodes)]
     log.debug("Retrieved nodelist: %s", node_list)
@@ -123,11 +154,16 @@ def _resume(arg_nodes, resume_config):
         hosted_zone=resume_config.hosted_zone,
         dns_domain=resume_config.dns_domain,
         use_private_hostname=resume_config.use_private_hostname,
-        master_private_ip=resume_config.master_private_ip,
-        master_hostname=resume_config.master_hostname,
+        head_node_private_ip=resume_config.head_node_private_ip,
+        head_node_hostname=resume_config.head_node_hostname,
         instance_name_type_mapping=resume_config.instance_name_type_mapping,
     )
-    instance_manager.add_instances_for_nodes(node_list, resume_config.max_batch_size, resume_config.update_node_address)
+    instance_manager.add_instances_for_nodes(
+        node_list=node_list,
+        launch_batch_size=resume_config.max_batch_size,
+        update_node_address=resume_config.update_node_address,
+        all_or_nothing_batch=resume_config.all_or_nothing_batch,
+    )
     success_nodes = [node for node in node_list if node not in instance_manager.failed_nodes]
     log.info("Successfully launched nodes %s", print_with_count(success_nodes))
     if instance_manager.failed_nodes:
@@ -139,6 +175,13 @@ def _resume(arg_nodes, resume_config):
 
 
 def main():
+    default_log_file = "/var/log/parallelcluster/slurm_resume.log"
+    logging.basicConfig(
+        filename=default_log_file,
+        level=logging.INFO,
+        format="%(asctime)s - [%(name)s:%(funcName)s] - %(levelname)s - %(message)s",
+    )
+    log.info("ResumeProgram startup.")
     parser = argparse.ArgumentParser()
     parser.add_argument("nodes", help="Nodes to burst")
     args = parser.parse_args()
@@ -148,18 +191,13 @@ def main():
             # Configure root logger
             fileConfig(resume_config.logging_config, disable_existing_loggers=False)
         except Exception as e:
-            default_log_file = "/var/log/parallelcluster/slurm_resume.log"
-            logging.basicConfig(
-                filename=default_log_file,
-                level=logging.INFO,
-                format="%(asctime)s - [%(name)s:%(funcName)s] - %(levelname)s - %(message)s",
-            )
             log.warning(
                 "Unable to configure logging from %s, using default settings and writing to %s.\nException: %s",
                 resume_config.logging_config,
                 default_log_file,
                 e,
             )
+        log.info("ResumeProgram config: %s", resume_config)
         _resume(args.nodes, resume_config)
         log.info("ResumeProgram finished.")
     except Exception as e:
